#!/bin/bash

# Built on 03:28 PM Monday, January 17, 2011

:<< _EOF 

This patch script is a part of the installation tar-ball.
It basically deprecates the use of install.sh for upgradation, when simple patching of the files is required

usage:
/etc/init.d/monit stop; /etc/init.d/safesquid stop; ./patch.sh ; /etc/init.d/safesquid start ; /etc/init.d/monit start;

_EOF

# identify our current location in the file system for future referencing
CWD=${PWD}
CUR_DATE=`date +%Y%m%d%H%M%S`
PATCH_LOG=${CWD}/safesquid-patch-${CUR_DATE}
echo "Recording the patching process in ${PATCH_LOG}"
touch ${PATCH_LOG}
ROLL_BACK=${CWD}/rollback-patch-${CUR_DATE}.sh

LOGIT ()
{
	while read -t 1 a
	do
		echo $a | tee -a ${PATCH_LOG}
	done
	
	[ "x$*" != "x" ] && echo $* | tee -a ${PATCH_LOG}

}

ROLL_BACK_HEADER ()
{
cat << _EOF >> ${ROLL_BACK}
#!/bin/bash
# This is an auto-generated bash shell script
# Executing this script will roll-back the patch operations carried out on `date "+%Y-%m-%d %H:%M:%S"` (YYYY-MM-DD HH:MM:SS)

	echo "Rolling back patch operations carried out on `date "+%Y-%m-%d %H:%M:%S"` (YYYY-MM-DD HH:MM:SS)"
	
_EOF

}

BIN_ROLL_BACK ()
{
cat << _EOF >> ${ROLL_BACK}


	echo "setting soft-link ${INSTALL_DIR}/bin/safesquid -> ${OLD_EXE}"
	ln -vfs ${OLD_EXE} ${INSTALL_DIR}/bin/safesquid 
_EOF

}

XML_ROLL_BACK ()
{
cat << _EOF >> ${ROLL_BACK}

	echo "Copying ${INSTALL_DIR}/bin/section-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}.xml -> ${INSTALL_DIR}/bin/section.xml"
	cp --preserve=mode,ownership,timestamp -v ${INSTALL_DIR}/bin/section-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}.xml ${INSTALL_DIR}/bin/section.xml

_EOF


}

MOD_ROLL_BACK ()
{
cat << _EOF >> ${ROLL_BACK}
	echo "Copying ${MODDIR}-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]} -> ${MODDIR} "
	cp --preserve=mode,ownership,timestamp -rv ${MODDIR}-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}  ${MODDIR}
	echo "Setting softlinks  ${MODDIR}/ -> ${MODDIR}/*/*.so"
	ln -vfs ${MODDIR}/*/*.so ${MODDIR}/
	ln -vfs ${MODDIR}/*/*.xml ${MODDIR}/
	
_EOF
}

# check if SafeSquid was installed previously

if [ ! -f /etc/init.d/safesquid ]
then
	LOGIT "Failed to locate the safesquid init script"
	LOGIT "SafeSquid Patching cannot be initialized"
	exit 1
fi


# we need to find out the installation folders, we must locate the startup.conf to get these
INIT_SCRIPT=`readlink -f /etc/init.d/safesquid`
STARTUP_CONF=`dirname ${INIT_SCRIPT}`/startup.conf

# read in the variables from the identified startup.conf
source ${STARTUP_CONF}

if [ "x${INSTALL_DIR}" == "x" ]
then
	LOGIT "Failed to detect Installation Directory of previously installed SafeSquid"
	exit 1
fi

OLD_EXE=`readlink -f ${INSTALL_DIR}/bin/safesquid`

if [ ! -f ${OLD_EXE} ] 
then
	LOGIT "Existing installation suggests a non-existant file at ${OLD_EXE} "
	exit 1
fi

ROLL_BACK_HEADER

LOGIT "Discovered ${OLD_EXE} as the currently set SafeSquid Binary Executable"
LOGIT "You are currently using:"
${OLD_EXE} -v | LOGIT
OLD_VER_ARRAY=( `${OLD_EXE} -v | awk 'BEGIN{FS="-"}END{print $3,$5}'` )

# locate the new executable extracted from the tar-ball
if [ -f bin/safesquid* ]
then
	NEW_EXE=`dir -1t ${CWD}/bin/safesquid* | head -n 1`
	LOGIT "Found ${NEW_EXE} in the patch package"
	# do the basic executation of the new executable and check the dependency libraries
	${NEW_EXE} -v | LOGIT
	ldd ${NEW_EXE}  | LOGIT
	NEW_BIN=${NEW_EXE/*\/}
	chmod -v ${NEW_EXE} --reference=${OLD_EXE} | LOGIT
	chown -v --reference=${OLD_EXE} ${NEW_EXE} | LOGIT
	LOGIT "patching new executable ${NEW_EXE} into INSTALL_DIR: ${INSTALL_DIR}"
	cp --preserve=mode,ownership -vf ${NEW_EXE} ${INSTALL_DIR}/bin/  | LOGIT
	ln -vfs ${INSTALL_DIR}/bin/${NEW_BIN} ${INSTALL_DIR}/bin/safesquid | LOGIT
	BIN_ROLL_BACK	
	
else
	LOGIT "No SafeSquid executable binary found in the patch package"
fi

if [ -f ${CWD}/bin/section.xml ]
then
	chmod -v ${CWD}/bin/section.xml --reference=${OLD_EXE} | LOGIT
	chown -v --reference=${OLD_EXE} ${CWD}/bin/section.xml | LOGIT
	LOGIT "Backing up existing ${INSTALL_DIR}/section.xml as ${INSTALL_DIR}/section-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}.xml"
	cp --preserve=mode,ownership -v ${INSTALL_DIR}/bin/section.xml ${INSTALL_DIR}/bin/section-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}.xml | LOGIT
	cp --preserve=mode,ownership ${CWD}/bin/section.xml ${INSTALL_DIR}/bin/section.xml
	XML_ROLL_BACK
else
	LOGIT "No section.xml found in the patch package"
fi

if [ -d ${CWD}/modules/ ] 
then
	chmod -Rv ${CWD}/modules/* --reference=${OLD_EXE} | LOGIT
	chown -Rv --reference=${OLD_EXE} ${CWD}/modules/* | LOGIT
	LOGIT "Backing up existing Add-On Modules Directory ${MODDIR} to ${MODDIR}-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]}"
	cp --preserve=mode,ownership -rv ${MODDIR} ${MODDIR}-${OLD_VER_ARRAY[0]}-${OLD_VER_ARRAY[1]} | LOGIT
	LOGIT "Copying New Add-On modules into ${MODDIR}"
	cp --preserve=mode,ownership -frv ${CWD}/modules/* ${MODDIR}/  | LOGIT
	LOGIT "Setting up soft-links for add-on modules"
	cd  ${MODDIR}/
	
	for SO in modules/*/*.so
	do 
		ln -vfs ${SO} `basename ${SO}` | LOGIT
	done
	
	for XML in modules/*/*.xml
	do 
		ln -vfs ${XML} `basename ${XML}` | LOGIT
	done

	cd ${CWD}
	MOD_ROLL_BACK
	
else
	LOGIT "No Add-on Modules found in the patch package"
fi

LOGIT "Patch completed"
LOGIT "You are now using:"
${INSTALL_DIR}/bin/safesquid -v  | LOGIT
chmod -v 755 ${ROLL_BACK}
LOGIT "Roll Back to previous version can be achieved by executing ${ROLL_BACK}"
chmod -v 755 ${ROLL_BACK} | LOGIT
echo "Logs of this patching operation stored in ${PATCH_LOG}"
